dojo.provide ("javascript.widgets.trouble.ChangeLogLevels");
dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.data.ItemFileReadStore");

dojo.declare("javascript.widgets.trouble.ChangeLogLevels", null, {
	
	constructor : function(resourceURL, logType){
		console.log("constructor of ChangeLogLevels entered with(resourceURL, logType): ", arguments)
		this.resourceURL = resourceURL;
		this.logType = logType;
	},
	getCurrentTraceSpecification : function(expandTraceLevels){
		console.log("entered getCurrentTraceSpecification(expandTraceLevels): ", expandTraceLevels);
		var scope = this;
		var request = {
			url: this.resourceURL,
			handleAs: "text",
			headers: {
                'Accept': 'application/rainmaker+trace+spec+view' 
            },
            content: { expandTraceLevels: expandTraceLevels },
			load: function (response, ioArgs){
				var tempDiv = document.createElement("div");
				// get the entire refreshed page (only UI or Data Cache portion), and place into a tempDiv
				tempDiv.innerHTML = response;
				// pick out the new div with the new data from the response, starts with div
				var divContentToReplace = dojo.query("[id=logging-configure-trace-levels-" + scope.logType +  "-section]", tempDiv);
				// get the inner table from the new div, and replace the old one with it (replaces table inside the div)
				dojo.byId('logging-configure-trace-levels-'+ scope.logType +'-section').innerHTML = divContentToReplace[0].innerHTML;
				// get a reference to the newly updated portion of the old div (with table)
				var traceSpecDiv = dojo.byId('logging-configure-trace-levels-'+ scope.logType +'-section');
				// make sure any widgets returned from the response get processed by dojo as widgets (so everything keeps
				// working as before)
				dojo.parser.parse(traceSpecDiv);
				// this ports functionality that was available in prototype, but not dojo, namely, this makes sure
				// any javascript returned in script tags get executed. This also ensure things behave after the response
				// as they did before the response
				Bedrock.evalScripts(response);
			},
			error: function(data, ioArgs){
				//do nothing
			}
		}
		var deferred = dojo.xhrGet(request);
		console.log("exiting getCurrentTraceSpecification");
    },
    
    updateTraceLevel : function (loggerId, loggerKey, value){
	    var scope = this;
		var obj = new Object();
        obj['level'] = value;
        console.log(" updateTraceLevel(loggerId, loggerKey, value): ", arguments);
        console.log(" obj should 3rd arg from above: ", obj);
        console.log(" url to do PUT: ", scope.resourceURL + '/' + loggerKey);
        var itemToPost= dojo.toJson(obj);
		var request = {
			url: scope.resourceURL +'/'+ loggerKey,
			handleAs: "text",
			contentType: "application/json" ,
			postData: itemToPost,
			load: function (response, ioArgs){
				//do nothing on success 
			},
			error: function(data, ioArgs){
		  		dojo.byId('errorTraceLevel').innerHTML = data;
			}
		}
		var deferred = dojo.rawXhrPut(request);
		console.log("exiting updateTraceLevel");
    },
    
	inlineTraceEditor : function(loggerId, loggerKey, currentValue, title, noValue){
		console.debug('entered inlineTraceEditor (loggerId, loggerKey, currentValue, title, noValue):', arguments);
		var priorEditor = dijit.byId(loggerId);
		if(priorEditor){
			priorEditor.disconnect();
			priorEditor.destroy();
		}
		currentValue = currentValue.toUpperCase();
		var standardLoggerLevels = [
							    { logLevel:'OFF'},
							    { logLevel:'SEVERE'},
							    { logLevel:'WARNING'},
							    { logLevel:'INFO'},
							    { logLevel:'FINE'},
							    { logLevel:'FINER'},
							    { logLevel:'FINEST'},
							    { logLevel:'ALL'} ];
		var defaultLoggerLogLevels = [
								{ logLevel:'OFF'},
							    { logLevel:'SEVERE'},
							    { logLevel:'WARNING'},
							    { logLevel:'INFO'}];
							    //,{ logLevel:'FINE'}];							    
		var loggerLevels = standardLoggerLevels;
		// if we are rendering the default logger, only allow limited trace specification
		// this prevents over burdening the appliance, and possibly bringing down xsa.app
		if(loggerKey == '*'){
			loggerLevels = defaultLoggerLogLevels;
		}
		
		var scope = this;
		var logStore = new dojo.data.ItemFileReadStore( { 
						data : { 
						  label: 'logLevel',
						  identifier: 'logLevel',
						  query: {logLevel: "*"},
						  items: loggerLevels
						}
					}
				);
		var eb = new dijit.InlineEditBox({
			editor: "dijit.form.FilteringSelect",
			editorParams: {store: logStore, autoComplete: true, hasDownArrow : true, searchAttr : 'logLevel', labelAttr : 'logLevel', labelType : 'html'},
			autoSave: true,
			noValueIndicator: noValue,
			title: title,
			style: "background-color: #ffffff;",
			'class': "inplace-form-table-editing",
			value: currentValue
		},
		loggerId);
		eb.connect(eb, "onChange", dojo.hitch(scope, scope.updateTraceLevel, loggerId, loggerKey));
		console.log("exiting inlineTraceEditor");
	},

	addTraceElement : function(noValueProvided, value) {
		console.log("entered addTraceElement with (noValueProvide, value):", arguments);
		if (value == noValueProvided)
			 return null;
		value = Bedrock.strip(value);
		if(value == ''){
			return null;
		}
		
		var obj = new Object();
		obj['logger'] = value;
		// colon delimitation might need to happen here
		// either we send several requests to sMash in a loop, or we
		// we let traceSpec.groovy handle it for us.
		var itemToPost = dojo.toJson(obj);

	    var scope = this;
	    // Here we are sending the requested new trace spec in the form of {logger: value} as JSON
	    // We are updating the config (temporary) zone holding the map of key=traceSpec, value=traceLevel
	    // values we need to read in the UI
	    // ex: {logger : "GridStats"}
	    // ex: {logger: "GridStats: Paul"}
		var request = {
			url: scope.resourceURL,
			handleAs: "text",
			contentType: "application/json",
			postData: itemToPost,
			load: function (response, ioArgs){
				scope.getCurrentTraceSpecification(true);
			},
			error: function(data, ioArgs){
		  		dojo.byId('errorTraceLevel-'+this.logType).innerHTML = data;
			}
		}
		var deferred = dojo.rawXhrPost(request);
		console.log("exiting addTraceElement");
	},
	
	enableTraceElement : function(loggerId, loggerValue) {
		console.log("entered enableTraceElement with (loggerId, loggerValue):", arguments);
		var obj = new Object();
		obj[loggerId] = loggerValue;
		// colon delimitation might need to happen here
		// either we send several requests to sMash in a loop, or we
		// we let traceSpec.groovy handle it for us.
		var itemToPost = dojo.toJson(obj);

	    var scope = this;
	    // Here we are sending the requested new trace spec in the form of {logger: value} as JSON
	    // We are updating the config (temporary) zone holding the map of key=traceSpec, value=traceLevel
	    // values we need to read in the UI
	    // ex: {logger : "GridStats"}
	    // ex: {logger: "GridStats: Paul"}
		var request = {
			url: scope.resourceURL,
			handleAs: "text",
			contentType: "application/json",
			postData: itemToPost,
			load: function (response, ioArgs){
				//scope.getCurrentTraceSpecification(true);
			},
			error: function(data, ioArgs){
		  		dojo.byId('errorTraceLevel-'+this.logType).innerHTML = data;
			}
		}
		var deferred = dojo.rawXhrPost(request);
		console.log("exiting enableTraceElement");
	},
	
	deleteTraceElement : function(loggerId, loggerKey) {
		console.log("entered deleteTraceElement with (loggerId, loggerKey):", arguments);
		var loggerInlineEditor = dijit.byId(loggerId);
		if(loggerInlineEditor){
			loggerInlineEditor.disconnect();
			loggerInlineEditor.destroy();
		}
	
	    var scope = this;
		var request = {
			url: scope.resourceURL +'/' + loggerKey,
			load: function (response, ioArgs){
				scope.getCurrentTraceSpecification(true);
			},
			error: function(data, ioArgs){
		  		dojo.byId('errorTraceLevel-' + this.logType).innerHTML = data;
			}
		}
		var deferred = dojo.xhrDelete(request);
		console.log("exiting deleteTraceElement");
	},
	
	disableTraceElement : function(loggerId, loggerLevel) {
		console.log("entered disableTraceElement with (htmlId, loggerId, loggerLevel):", arguments);
	
	    var scope = this;
		var request = {
			url: scope.resourceURL +'/' + loggerId,
			load: function (response, ioArgs){
				//scope.getCurrentTraceSpecification(true);
			},
			error: function(data, ioArgs){
		  		dojo.byId('errorTraceLevel-' + this.logType).innerHTML = data;
			}
		}
		var deferred = dojo.xhrDelete(request);
		console.log("exiting disableTraceElement");
	},
	
	addLoggingCheckboxListener : function (htmlId, loggerId, loggerLevel, values) {
		//this.defaultValues[htmlId] = dojo.byId(htmlId).checked;
		var scope=this;
		//var tooltip = this._addEthernetTooltip(htmlId);		
		dojo.connect(dojo.byId(htmlId), "onclick", dojo.hitch(scope, scope._checkboxEntryChange, htmlId, loggerId, loggerLevel, values));
	},
	
	_checkboxEntryChange : function (htmlId, loggerId, loggerLevel, values){
		var scope = this;
	    if (dojo.byId(htmlId).checked == true) {
	    	scope.enableTraceElement(loggerId, loggerLevel);
	    } else {
	    	scope.disableTraceElement(loggerId, loggerLevel);
	    }
	},		
    
	createTraceElementEditor : function (fieldName, title, noValue){
		console.log("entered createTraceElement with (fieldName, title, noValue):", arguments);
		var priorEditor = dijit.byId(fieldName);
		if(priorEditor){
			priorEditor.disconnect();
			priorEditor.destroy();
		}
		var scope = this;
		var eb = new dijit.InlineEditBox({
			editor: "dijit.form.TextBox",
			autoSave: true,
			noValueIndicator: noValue,
			title: title,
			style: "background-color: #ffffff;",
			'class': "inplace-form-table"
		},
		fieldName);
		eb.connect(eb, "onChange", dojo.hitch(scope, scope.addTraceElement, noValue));
		console.log("exiting createTraceElmentEditor");
	}
});		
